@@ -849,20 +849,18 @@ static int qmgr_resolve_one(QMGR_MESSAGE *message, RECIPIENT *recipient,
     DSN     dsn;
 
     if ((message->tflags & DEL_REQ_FLAG_MTA_VRFY) == 0)
-	resolve_clnt_query(addr, reply);
+	resolve_clnt_query_from(message->sender, addr, reply);
     else
-	resolve_clnt_verify(addr, reply);
+	resolve_clnt_verify_from(message->sender, addr, reply);
     if (reply->flags & RESOLVE_FLAG_FAIL) {
 	qmgr_defer_recipient(message, recipient,
-			     DSN_SMTP(&dsn, "4.3.0",
-				      "451 address resolver failure",
-				      "address resolver failure"));
+			     DSN_SIMPLE(&dsn, "4.3.0",
+					"address resolver failure"));
 	return (-1);
     } else if (reply->flags & RESOLVE_FLAG_ERROR) {
 	qmgr_bounce_recipient(message, recipient,
-			      DSN_SMTP(&dsn, "5.1.3",
-				       "553 bad address syntax",
-				       "bad address syntax"));
+			      DSN_SIMPLE(&dsn, "5.1.3",
+					 "bad address syntax"));
 	return (-1);
     } else {
 	return (0);
@@ -920,6 +918,7 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 	 * Content filtering overrides the address resolver.
 	 */
 	else if (message->filter_xport) {
+	    reply.flags = 0;
 	    vstring_strcpy(reply.transport, message->filter_xport);
 	    if ((nexthop = split_at(STR(reply.transport), ':')) == 0
 		|| *nexthop == 0)
@@ -932,7 +931,7 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 	 * Resolve the destination to (transport, nexthop, address). The
 	 * result address may differ from the one specified by the sender.
 	 */
-	else if (var_sender_routing == 0) {
+	else {
 	    if (qmgr_resolve_one(message, recipient,
 				 recipient->address, &reply) < 0)
 		continue;
@@ -940,17 +939,6 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 		RECIPIENT_UPDATE(recipient->address, STR(reply.recipient));
 	}
 
-	/*
-	 * XXX Sender-based routing does not work very well, because it has
-	 * problems with sending bounces.
-	 */
-	else {
-	    if (qmgr_resolve_one(message, recipient,
-				 message->sender, &reply) < 0)
-		continue;
-	    vstring_strcpy(reply.recipient, recipient->address);
-	}
-
 	/*
 	 * Bounce null recipients. This should never happen, but is most
 	 * likely the result of a fault in a different program, so aborting
@@ -958,9 +946,8 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 	 */
 	if (recipient->address[0] == 0) {
 	    qmgr_bounce_recipient(message, recipient,
-				  DSN_SMTP(&dsn, "5.1.3",
-					   "553 null recipient address",
-					   "null recipient address"));
+				  DSN_SIMPLE(&dsn, "5.1.3",
+					     "null recipient address"));
 	    continue;
 	}
 
@@ -977,9 +964,8 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 	 */
 	if (var_allow_min_user == 0 && recipient->address[0] == '-') {
 	    qmgr_bounce_recipient(message, recipient,
-				  DSN_SMTP(&dsn, "5.1.3",
-					   "553 bad address syntax",
-					   "bad address syntax"));
+				  DSN_SIMPLE(&dsn, "5.1.3",
+					     "bad address syntax"));
 	    continue;
 	}
 
@@ -1025,9 +1011,8 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 		    break;
 	    if (*cpp) {
 		qmgr_defer_recipient(message, recipient,
-				     DSN_SMTP(&dsn, "4.3.2",
-					      "450 delivery suspended",
-					      "deferred transport"));
+				     DSN_SIMPLE(&dsn, "4.3.2",
+						"deferred transport"));
 		continue;
 	    }
 	}
